//
//  File SScheduler.C
//
//  November 1994 - March 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "SScheduler.H"
#include "Synchronous.H"
#include "Signal.H"
#include "Array.H"
#include "QSort.H"
#include "Connect.H"

// SScheduler
//

SScheduler::SScheduler(const char* const name,
                     const bool simul,
                     const bool debug)
          : dirty(false),
            iShouldStop(false),
            fName(name),
            fSimul(simul),
            fDebug(debug)
{
  theScheduler = this;     // Make this new scheduler the clock
}

SScheduler::~SScheduler() {
  for(int i = 0; i < connections.size(); i++)
      delete connections[i];

  if (theScheduler == this)
    theScheduler = 0;
}

void SScheduler::manage(Synchronous *s) {
  newModules[newModules.size()] = s;
  if (fDebug) {
    cerr << "..." << fName << ".manage(" << s->name() << ")" << endl;
  }
  dirty = true;
}

void SScheduler::trash(Synchronous *s) {
  delModules[delModules.size()] = s;
  if (fDebug) {
    cerr << "..." << fName << ".trash(" << s->name() << ")" << endl;
  }
  dirty = true;
}

void SScheduler::connect(InputSignal *f, OutputSignal *t) {
  connections[connections.size()] = new Connect(f, t);
  if (fDebug) {
    cerr << "..." << fName << ".connect(" << *f << ", " << *t << ")" << endl;
  }
  dirty = true;
}

void SScheduler::backup() {
  if (fDebug) {
    cerr << "..." << fName << " --- Backup." << endl;
  }
  if (dirty)
    cleanup();
  for(int i = 0; i < modules.size(); i++)
    modules[i]->backup();
}

void SScheduler::restore() {
  if (fDebug) {
    cerr << "..." << fName << " --- Restore." << endl;
  }
  if (dirty)
    cleanup();
  for(int i = 0; i < modules.size(); i++)
    modules[i]->restore();
}

void SScheduler::stop() {
  iShouldStop = true;
}

void SScheduler::cleanup() {
  int i;
  
  if (fDebug) {
	cerr << "..." << fName << " is dirty" << endl;
  }
  
  for(i = 0; i < delModules.size(); i++) {
    delModules[i]->die();
    for(int j = 0; j < modules.size(); j++) {
      if (modules[j] == delModules[i]) {
        modules[j] = 0;
        break;
      }
    }
  }
  delModules.empty();
  
  for(i = 0; i < newModules.size(); i++)
    modules[modules.firstFree()] = newModules[i];
  newModules.empty();

  for(i = 0; i < connections.size(); i++)
    connections[i]->doConnect();
  connections.empty();

  QSort<Synchronous>(modules).sort();
  modules.crop();
  
  dirty = false;
}

bool SScheduler::tick() {
  char c;
  int i;
  
  if (fSimul)
    cerr << "..." << fName << " --- Start of instant." << endl;
  if (dirty) {
    cleanup();
  }
  if (fDebug) {
    cerr << "..." << fName << " : Modules in this instant:" << endl;
    for (i = 0; i < modules.size(); i++)
      cerr << "......" << modules[i]->name() << endl;
  }
  for(i = 0; i < modules.size(); i++)
    modules[i]->resetOutputs();

  for(i = 0; i < modules.size(); i++)
    modules[i]->resetInputs();

  for(i = 0; i < modules.size(); i++)
    modules[i]->react();

  for(i = 0; i < modules.size(); i++)
    modules[i]->endInstant();
  
  if (fSimul) {
    cerr << "..." << fName << " --- End of instant. (c)ontinue, (s)top : " << flush;
    c = cin.get();
    switch (c) {
      case 's':
      case 'S':
        iShouldStop = true;
        while((c = cin.get()) != '\n')
          ;
        break;
      case '\n':
        break;
      default:
        while((c = cin.get()) != '\n')
          ;
        break;
    }
  }
  if (dirty) {
    cleanup();
  }
  return (!iShouldStop);
}

SScheduler *theScheduler = 0;  // Global scheduler.

// End of file SScheduler.C
