//
//  File TimeModule.C
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91192 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "TimeModule.H"

#ifdef __osf__
#include <sys/timers.h>
extern "C" int getclock(int clktyp, struct timespec *tp);
#endif

TimeModule::TimeModule(const char* name, const bool schedule)
          : Synchronous(name, schedule),
            fMilliSecond(this,"milliSecond"),
            fCentiSecond(this,"centiSecond"),
            fDeciSecond(this,"deciSecond"),
            fSecond(this,"second"),
            fMinute(this,"minute"),
            fHour(this,"hour"),
            fBackuping(false), fRestoring(false),
            pf_backup_copy_(0) {
  fMilliSecond.value() = -1;
  fCentiSecond.value() = -1;
  fDeciSecond.value() = -1;
  fSecond.value() = -1;        // Impossible values to make
  fMinute.value() = -1;        // sure these signals will be
  fHour.value() = -1;          // emitted at first activation.
}

TimeModule::~TimeModule() {
  if (pf_backup_copy_ != 0)
    delete pf_backup_copy_;
}

const InputSignal** TimeModule::inSigList() const {
  const InputSignal** inList;

  inList = (const InputSignal**) new (InputSignal*[1]);
  inList[0] = 0;
  return inList;
}

const OutputSignal** TimeModule::outSigList() const {
  const OutputSignal** sigList;

  sigList = (const OutputSignal**) new (OutputSignal*[7]);
  sigList[0] = &fMilliSecond;
  sigList[1] = &fCentiSecond;
  sigList[2] = &fDeciSecond;
  sigList[3] = &fSecond;
  sigList[4] = &fMinute;
  sigList[5] = &fHour;
  sigList[6] = 0;
  return sigList;
}

OutSignal<int>& TimeModule::milliSecond() {
  return fMilliSecond;
}

OutSignal<int>& TimeModule::centiSecond() {
  return fCentiSecond;
}

OutSignal<int>& TimeModule::deciSecond() {
  return fDeciSecond;
}

OutSignal<int>& TimeModule::second() {
  return fSecond;
}

OutSignal<int>& TimeModule::minute() {
  return fMinute;
}

OutSignal<int>& TimeModule::hour() {
  return fHour;
}

void TimeModule::resetOutputs() {
  fMilliSecond.value().reset();
  fDeciSecond.value().reset();
  fSecond.value().reset();
  fMinute.value().reset();
  fHour.value().reset();
}

void TimeModule::setInputs() {
  if(fRestoring) {
  	fMilliSecond.value() = pf_backup_copy_->fMilliSecond;
  	fCentiSecond.value() = pf_backup_copy_->fCentiSecond;
  	fDeciSecond.value() = pf_backup_copy_->fDeciSecond;
  	fSecond.value() = pf_backup_copy_->fSecond;
  	fMinute.value() = pf_backup_copy_->fMinute;
  	fHour.value() = pf_backup_copy_->fHour;
    fRestoring = false;
    return;
  }
#ifdef __osf__
  getclock(TIMEOFDAY,&fClock);  // Get the system clock
#else
  ftime(&fClock);  // Get the system clock
#endif
  if (fBackuping) {
  	pf_backup_copy_->fMilliSecond = fMilliSecond.value();
    pf_backup_copy_->fCentiSecond = fCentiSecond.value();
    
  	pf_backup_copy_->fDeciSecond = fDeciSecond.value();
  	pf_backup_copy_->fSecond = fSecond.value();
  	pf_backup_copy_->fMinute = fMinute.value();
  	pf_backup_copy_->fHour = fHour.value();
    fBackuping = false;
  }
}

void TimeModule::resetInputs() {
}

void TimeModule::activate() {
  struct tm *theTime;
#ifdef __osf__
  theTime = localtime(&(fClock.tv_sec));  // Convert clock to the current time
  // Now, test if values have changed, and emit signals that changed.
  if ((fClock.tv_nsec / 1000000) != fMilliSecond.value().value())
  fMilliSecond.value() = int(fClock.tv_nsec/1000000); // Type cast to int for g++

  if ((fClock.tv_nsec / 10000000) != fCentiSecond.value().value())
  fCentiSecond.value() = fClock.tv_nsec / 10000000;

  if ((fClock.tv_nsec / 100000000) != fDeciSecond.value().value())
  fDeciSecond.value() = fClock.tv_nsec / 100000000;
#else
  theTime = localtime(&(fClock.time));  // Convert clock to the current time
  // Now, test if values have changed, and emit signals that changed.
  if (fClock.millitm != fMilliSecond.value().value())
  fMilliSecond.value() = int(fClock.millitm); // Type cast to int for g++

  if ((fClock.millitm / 10) != fCentiSecond.value().value())
  fCentiSecond.value() = fClock.millitm / 10;

  if ((fClock.millitm / 100) != fDeciSecond.value().value())
  fDeciSecond.value() = fClock.millitm / 100;
#endif
  if (theTime->tm_sec != fSecond.value().value())
  fSecond.value() = theTime->tm_sec;

  if (theTime->tm_min != fMinute.value().value())
  fMinute.value() = theTime->tm_min;

  if (theTime->tm_hour != fHour.value().value())
  fHour.value() = theTime->tm_hour;
}

void TimeModule::backup() {
  if (pf_backup_copy_ == 0) {
    pf_backup_copy_ = new TimeData;
  }
  fBackuping = true;
}

void TimeModule::restore() {
  if (!fBackuping && (pf_backup_copy_ != 0)) {
    fRestoring = true;
  }
}

//TimeModule& TimeModule::operator=(const TimeModule& m) {
//  fMilliSecond.value() = ((TimeModule&)m).fMilliSecond.value();
//  fCentiSecond.value() = ((TimeModule&)m).fCentiSecond.value();
//  fDeciSecond.value() = ((TimeModule&)m).fDeciSecond.value();
//  fSecond.value() = ((TimeModule&)m).fSecond.value();
//  fMinute.value() = ((TimeModule&)m).fMinute.value();
//  fHour.value() = ((TimeModule&)m).fHour.value();
//  fClock = m.fClock;
//  return *this;
//}
