//
//  File coreIOModule.C
//
//  September 3 1992 - November 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "IOModule.H"

// InputModule
//

PureInModule::PureInModule(const char *prompt, const char *tail,
                           const bool errNotPres,
                           const char* name, const bool schedule)
             : Synchronous(name, schedule),
               fInputStream(&cin),
               fOutputStream(&cout),
               fLogStream(&nullDev),
               fPrompt(prompt),
               fTail(tail),
               fInput(this, "Input"), 
               ignoreErrs(errNotPres),
               fRestoring(false),
               fBackuping(false),
               pf_backup_copy_(0) {
}

void PureInModule::setStreams(istream& i, ostream& o, ostream& l) {
  fInputStream = &i;
  fOutputStream = &o;
  fLogStream = &l;
}

PureOutSignal& PureInModule::input() {
  return fInput;
}

PureOutSignal& PureInModule::operator()() {
  return fInput;
}

const InputSignal** PureInModule::inSigList() const {
  const InputSignal** inList;

  inList = (const InputSignal**) new (InputSignal*[1]);
  inList[0] = 0;
  return inList;
}

const OutputSignal** PureInModule::outSigList() const {
  const OutputSignal** sigList;

  sigList = (const OutputSignal**) new (OutputSignal*[2]);
  sigList[0] = &fInput;
  sigList[1] = 0;
  return sigList;
}

void PureInModule::resetOutputs() {
  fInput.value().reset();
}

void PureInModule::setInputs() {
  if (fRestoring) {
    fInput.value() = pf_backup_copy_->value();
    fRestoring = false;
    return;
  }
  bool done = false;
  while (!done) {
    *fOutputStream << fPrompt << flush;
    *fInputStream >> fInput.value();
    if (!fInputStream->good()) {
      if (ignoreErrs) {
        *fOutputStream << "### Bad value, ignored." << endl;
        done = true;
      } else {
        fInputStream->clear(0);
        *fOutputStream << "### Bad value, redo from start." << endl;
      }
    } else
      done = true;
    if (fInputStream->good()) {
      while (fInputStream->get() != '\n')
        ;
    }
  }
  *fOutputStream << fTail << flush;
  *fLogStream << fPrompt << " <- ";
  *fLogStream << fInput.value();
  *fLogStream << fTail << endl << flush;
  if (fBackuping) {
    pf_backup_copy_->value() = fInput.value();
    fBackuping = false;
  }
}

void PureInModule::resetInputs() {
}

void PureInModule::activate() {
}

void PureInModule::backup() {
  if (pf_backup_copy_ == 0)
    pf_backup_copy_ = new PureOutSignal(this,"backup");
  fBackuping = true;
}

void PureInModule::restore() {
  if (!fBackuping && (pf_backup_copy_ != 0))
    fRestoring = true;
}

// OutputModule
//
PureOutModule::PureOutModule(const char *banner, const bool always,
                                    const char *tail, const char* name,
                                    const bool schedule)
                      : Synchronous(name, schedule),
                        fOutputStream(&cout),
                        fLogStream(&nullDev),
                        fBanner(banner),
                        fTail(tail),
                        fAlways(always),
                        fProbe(this, "Probe") {
}

void PureOutModule::setStreams(ostream& o, ostream& l) {
  fOutputStream = &o;
  fLogStream = &l;
}

PureInSignal& PureOutModule::probe() {
  return fProbe;
}

PureInSignal& PureOutModule::operator()() {
  return fProbe;
}

const InputSignal** PureOutModule::inSigList() const {
  const InputSignal** inList;

  inList = (const InputSignal**) new (InputSignal*[2]);
  inList[0] = &fProbe;
  inList[1] = 0;
  return inList;
}

const OutputSignal** PureOutModule::outSigList() const {
  const OutputSignal** sigList;

  sigList = (const OutputSignal**) new (OutputSignal*[1]);
  sigList[0] = 0;
  return sigList;
}

void PureOutModule::resetOutputs() {
}

void PureOutModule::setInputs() {
  fProbe.update();
}

void PureOutModule::resetInputs() {
  fProbe.value().reset();
}

void PureOutModule::activate() {
  if ((fAlways)||(fProbe.value().isPresent())) {
    *fOutputStream << fBanner;
    *fOutputStream << fProbe.value();
    *fOutputStream << fTail << flush;
    *fLogStream << fBanner;
    *fLogStream<< fProbe.value();
    *fLogStream<< fTail << flush;
  }
}

void PureOutModule::backup() {
}

void PureOutModule::restore() {
}

// End of file coreIOModule.C

