//
//  File coreSignal.C
//
//  September 3 1992 - November 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Signal.H"
#include "Synchronous.H"
#include "SScheduler.H"

#include <iostream.h>

// Core Signal definitions
//

VSignal::VSignal() {
}

VSignal::VSignal(const Synchronous* s, const char* name)
        : fOwner(s), fName(name) {
}

const char* VSignal::name() const {
  return fName;
}

void VSignal::chown(const Synchronous* s) {
  fOwner = s;
}

void VSignal::rename(const char *newname) {
	fName = newname;
}

ostream& operator<<(ostream& o, const VSignal& s) {
  if (s.fOwner != 0)
    o << s.fOwner->name() << "." << s.name();
  else
    o << "<no owner>." << s.name();
  return o;
}

InputSignal::InputSignal(const Synchronous* s, const char* name,
                         const bool dep)
            : VSignal(s, name), source(0), dep(dep) {
}

bool InputSignal::checkLoopThrough(const OutputSignal& o) const {
  if (o.isDead())         // If signal is dead, no loop!
    return false;
  if (dep)
    return o.checkLoopFor(fOwner);
  else
    return false;
}

bool InputSignal::checkLoopFor(const Synchronous* s) const {
  if ((source == 0) || (source->isDead())) // No source or dead source, no loop
    return false;
  if (dep)                  // If I create a dependency
    return source->checkLoopFor(s); // Check for loop
  else
    return false;           // I do not create a dependency, so no loop!
}

void InputSignal::unplug() {
  if (source != 0)
    source->deccref();
  source = 0;
}

void InputSignal::checkSource() {
  if ((source != 0) && (source->isDead()))
    unplug();
}

void InputSignal::connectTo(OutputSignal* s, const bool doIt) {
  if (s == 0)
  	return;
  SScheduler *clock;
  if (fOwner != 0)
    clock = fOwner->clock();
  else
    clock = 0;
  if ((doIt) || (clock == 0)){
    if (checkLoopThrough(*s)) {
      cerr << "# Loop detected while attempting to connect " << *this
           << " to " << *s << endl;
      cerr << "## Connection aborted !" << endl;
    }else {
      unplug();
      source = s->inccref();
    }
  } else
    clock->connect(this, s);
}

OutputSignal::OutputSignal(const Synchronous* s, const char* name,
                           const bool dep)
            : VSignal(s, name), dep(dep), refcount(0) {
}

bool OutputSignal::checkLoopFor(const Synchronous* s) const {
  const InputSignal **inList;
  int i;

  if (isDead())
    return false;

  if (dep) {
    if (fOwner == s)
      return true;
    else {
      if (fOwner == 0)
        return false;
      inList = fOwner->inSigList();
      i = 0;
      while(inList[i] != 0) {
        if (inList[i]->checkLoopFor(s)) {
          delete[] inList;
          return true;
        }else
          i++;
      }
      delete[] inList;
      return false;
    }
  }else
    return false;
}

unsigned long OutputSignal::cref() const {
  return refcount;
}

OutputSignal* OutputSignal::inccref() {
  refcount++;
  return this;
}

void OutputSignal::deccref() {
  if (refcount > 0)
    refcount--;
  else
    cerr << "# Warning: Output signal " << *this << " has negative ref count" << endl;
}

bool OutputSignal::isDead() const {
  if (fOwner == 0)
    return false;
  else
    return fOwner->isDead();
}

PureSignal::PureSignal(const Synchronous* s, const char* name)
           : VSignal(s, name) {
}

PureSample& PureSignal::value() {
  return fValue;
}

PureInSignal::PureInSignal(const Synchronous* s, const char* name,
                           const bool dep)
            : PureSignal(s, name),
              InputSignal(s, name, dep),
              VSignal(s, name) {
}

void PureInSignal::operator<<(PureOutSignal& s) {
  connectTo(&s);
}

void PureInSignal::update() {
  if ((source != 0) && (!source->isDead())) // If signal is connected
    fValue = ((PureOutSignal*)source)->value();
                                // get next sample from source
  else {
    if (source != 0)            // If source is dead
      source = 0;               // Forget about it
  }
}

PureOutSignal::PureOutSignal(const Synchronous *s, const char* name,
                             const bool dep)
              : PureSignal(s, name),
                OutputSignal(s, name, dep),
                VSignal(s, name) {
}

// End of file coreSignal.C
